@@ -478,17 +478,9 @@ static BOOL logon_object_get_calculated_property(const logon_object *plogon,
 		std::string dispname;
 		if (!mysql_adaptor_get_user_displayname(plogon->account, dispname))
 			return FALSE;	
-		auto temp_len = utf8_to_mb_len(dispname.c_str());
-		auto tstr = cu_alloc<char>(std::max(temp_len, strlen(plogon->account) + 1));
-		*ppvalue = tstr;
-		if (*ppvalue == nullptr)
-			return FALSE;
-		if (common_util_convert_string(false, dispname.c_str(),
-		    tstr, temp_len) < 0)
-			return FALSE;	
-		if (*tstr == '\0')
-			strcpy(tstr, plogon->account);
-		return TRUE;
+		*ppvalue = !dispname.empty() ? cu_utf8_to_mb_dup(CP_OEMCP, dispname) :
+		           cu_utf8_to_mb_dup(CP_OEMCP, plogon->account);
+		return *ppvalue != nullptr ? TRUE : false;
 	}
 	case PR_MAX_SUBMIT_MESSAGE_SIZE: {
 		auto v = cu_alloc<uint32_t>();
@@ -563,7 +555,7 @@ static BOOL logon_object_get_calculated_property(const logon_object *plogon,
  * PR_HIERARCHY_SERVER property on the store visible with MFCMAPI, which means
  * EXC2019 also does not synthesize anything server-side in any way.
  */
-BOOL logon_object::get_properties(const PROPTAG_ARRAY *pproptags,
+bool logon_object::get_properties(proptag_cspan pproptags,
     TPROPVAL_ARRAY *ppropvals) const
 {
 	static const uint32_t err_code = ecError, invalid_code = ecInvalidParam;
@@ -571,15 +563,15 @@ BOOL logon_object::get_properties(const PROPTAG_ARRAY *pproptags,
 	auto pinfo = emsmdb_interface_get_emsmdb_info();
 	if (pinfo == nullptr)
 		return FALSE;
-	ppropvals->ppropval = cu_alloc<TAGGED_PROPVAL>(pproptags->count);
+	ppropvals->ppropval = cu_alloc<TAGGED_PROPVAL>(pproptags.size());
 	if (ppropvals->ppropval == nullptr)
 		return FALSE;
-	PROPTAG_ARRAY tmp_proptags = {0, cu_alloc<proptag_t>(pproptags->count)};
+	PROPTAG_ARRAY tmp_proptags = {0, cu_alloc<proptag_t>(pproptags.size())};
 	if (tmp_proptags.pproptag == nullptr)
 		return FALSE;
 	ppropvals->count = 0;
 	auto plogon = this;
-	for (const auto tag : *pproptags) {
+	for (const auto tag : pproptags) {
 		void *pvalue = nullptr;
 
 		if (PROP_ID(tag) == PROP_ID(PR_HIERARCHY_SERVER))
@@ -596,7 +588,7 @@ BOOL logon_object::get_properties(const PROPTAG_ARRAY *pproptags,
 
 	TPROPVAL_ARRAY tmp_propvals;
 	if (!exmdb_client->get_store_properties(plogon->dir,
-	    pinfo->cpid, &tmp_proptags, &tmp_propvals))
+	    pinfo->cpid, tmp_proptags, &tmp_propvals))
 		return FALSE;	
 	if (tmp_propvals.count == 0)
 		return TRUE;
@@ -647,19 +639,19 @@ BOOL logon_object::set_properties(const TPROPVAL_ARRAY *ppropvals,
 	return false;
 }
 
-BOOL logon_object::remove_properties(const PROPTAG_ARRAY *pproptags,
+bool logon_object::remove_properties(proptag_cspan pproptags,
     PROBLEM_ARRAY *pproblems)
 {
 	pproblems->count = 0;
-	pproblems->pproblem = cu_alloc<PROPERTY_PROBLEM>(pproptags->count);
+	pproblems->pproblem = cu_alloc<PROPERTY_PROBLEM>(pproptags.size());
 	if (pproblems->pproblem == nullptr)
 		return FALSE;
-	PROPTAG_ARRAY tmp_proptags = {0, cu_alloc<proptag_t>(pproptags->count)};
+	PROPTAG_ARRAY tmp_proptags = {0, cu_alloc<proptag_t>(pproptags.size())};
 	if (tmp_proptags.pproptag == nullptr)
 		return FALSE;
 	auto plogon = this;
-	for (unsigned int i = 0; i < pproptags->count; ++i) {
-		const auto tag = pproptags->pproptag[i];
+	for (unsigned int i = 0; i < pproptags.size(); ++i) {
+		const auto tag = pproptags[i];
 		if (lo_is_readonly_prop(plogon, tag))
 			pproblems->emplace_back(i, tag, ecAccessDenied);
 		else
@@ -667,7 +659,7 @@ BOOL logon_object::remove_properties(const PROPTAG_ARRAY *pproptags,
 	}
 	if (tmp_proptags.count == 0)
 		return TRUE;
-	return exmdb_client->remove_store_properties(plogon->dir, &tmp_proptags);
+	return exmdb_client->remove_store_properties(plogon->dir, tmp_proptags);
 }
 
 /**
