@@ -37,7 +37,6 @@ static EID_ARRAY *oxcfxics_load_folder_messages(logon_object *plogon,
 	uint32_t table_id;
 	uint32_t row_count;
 	TARRAY_SET tmp_set;
-	PROPTAG_ARRAY proptags;
 	RESTRICTION restriction;
 	EID_ARRAY *pmessage_ids;
 	RESTRICTION_PROPERTY res_prop;
@@ -54,10 +53,8 @@ static EID_ARRAY *oxcfxics_load_folder_messages(logon_object *plogon,
 	    &table_id, &row_count))
 		return NULL;	
 	proptag_t tmp_proptag = PidTagMid;
-	proptags.count = 1;
-	proptags.pproptag = &tmp_proptag;
 	if (!exmdb_client->query_table(plogon->get_dir(), nullptr, CP_ACP,
-	    table_id, &proptags, 0, row_count, &tmp_set))
+	    table_id, {&tmp_proptag, 1}, 0, row_count, &tmp_set))
 		return NULL;	
 	exmdb_client->unload_table(plogon->get_dir(), table_id);
 	pmessage_ids = eid_array_init();
@@ -103,7 +100,7 @@ oxcfxics_load_folder_content(logon_object *plogon, uint64_t folder_id,
 		return NULL;
 	auto pinfo = emsmdb_interface_get_emsmdb_info();
 	if (!exmdb_client->get_folder_properties(plogon->get_dir(), pinfo->cpid,
-	    folder_id, &tmp_proptags, &tmp_propvals))
+	    folder_id, tmp_proptags, &tmp_propvals))
 		return NULL;
 	auto pproplist = pfldctnt->get_proplist();
 	for (const auto &pv : tmp_propvals)
@@ -136,10 +133,8 @@ oxcfxics_load_folder_content(logon_object *plogon, uint64_t folder_id,
 	    &table_id, &row_count))
 		return NULL;
 	proptag_t tmp_proptag = PidTagFolderId;
-	tmp_proptags.count = 1;
-	tmp_proptags.pproptag = &tmp_proptag;
 	if (!exmdb_client->query_table(plogon->get_dir(), nullptr, CP_ACP,
-	    table_id, &tmp_proptags, 0, row_count, &tmp_set))
+	    table_id, {&tmp_proptag, 1}, 0, row_count, &tmp_set))
 		return NULL;
 	exmdb_client->unload_table(plogon->get_dir(), table_id);
 	for (const auto &row : tmp_set) {
@@ -203,13 +198,11 @@ ec_error_t rop_fasttransferdestconfigure(uint8_t source_operation, uint8_t flags
 	if (ROOT_ELEMENT_TOPFOLDER == root_element ||
 		ROOT_ELEMENT_MESSAGELIST == root_element ||
 		ROOT_ELEMENT_FOLDERCONTENT == root_element) {
-		static constexpr proptag_t proptag_buff[] =
+		static constexpr proptag_t tmp_proptags[] =
 			{PR_MESSAGE_SIZE_EXTENDED, PR_STORAGE_QUOTA_LIMIT,
 			PR_ASSOC_CONTENT_COUNT, PR_CONTENT_COUNT};
-		static constexpr PROPTAG_ARRAY tmp_proptags =
-			{std::size(proptag_buff), deconst(proptag_buff)};
 		TPROPVAL_ARRAY tmp_propvals;
-		if (!plogon->get_properties(&tmp_proptags, &tmp_propvals))
+		if (!plogon->get_properties(tmp_proptags, &tmp_propvals))
 			return ecError;
 
 		auto num = tmp_propvals.get<const uint32_t>(PR_STORAGE_QUOTA_LIMIT);
@@ -357,7 +350,7 @@ ec_error_t rop_fasttransfersourcecopyfolder(uint8_t flags, uint8_t send_options,
 	return ecSuccess;
 }
 
-ec_error_t rop_fasttransfersourcecopymessages(const LONGLONG_ARRAY *pmessage_ids,
+ec_error_t rop_fasttransfersourcecopymessages(const EID_ARRAY *pmessage_ids,
     uint8_t flags, uint8_t send_options, LOGMAP *plogmap, uint8_t logon_id,
     uint32_t hin, uint32_t *phout)
 {
@@ -396,7 +389,7 @@ ec_error_t rop_fasttransfersourcecopymessages(const LONGLONG_ARRAY *pmessage_ids
 	if (pmids == nullptr)
 		return ecServerOOM;
 	if (!eid_array_batch_append(pmids, pmessage_ids->count,
-	    pmessage_ids->pll)) {
+	    pmessage_ids->pids)) {
 		eid_array_free(pmids);
 		return ecServerOOM;
 	}
@@ -437,7 +430,7 @@ ec_error_t rop_fasttransfersourcecopymessages(const LONGLONG_ARRAY *pmessage_ids
  * CopyMessages cannot do that.
  */
 ec_error_t rop_fasttransfersourcecopyto(uint8_t level, uint32_t flags,
-    uint8_t send_options, const PROPTAG_ARRAY *pproptags, LOGMAP *plogmap,
+    uint8_t send_options, proptag_cspan pproptags, LOGMAP *plogmap,
     uint8_t logon_id, uint32_t hin, uint32_t *phout)
 {
 	ems_objtype object_type;
@@ -464,7 +457,7 @@ ec_error_t rop_fasttransfersourcecopyto(uint8_t level, uint32_t flags,
 		return ecError;
 	switch (object_type) {
 	case ems_objtype::folder: {
-		auto bg = pproptags->begin(), end = pproptags->end();
+		auto bg = pproptags.begin(), end = pproptags.end();
 		auto b_sub    = level == 0 && std::find(bg, end, PR_CONTAINER_HIERARCHY) == end;
 		auto b_fai    = level == 0 && std::find(bg, end, PR_CONTAINER_CONTENTS) == end;
 		auto b_normal = level == 0 && std::find(bg, end, PR_FOLDER_ASSOCIATED_CONTENTS) == end;
@@ -474,7 +467,7 @@ ec_error_t rop_fasttransfersourcecopyto(uint8_t level, uint32_t flags,
 		if (pfldctnt == nullptr)
 			return ecError;
 		auto pproplist = pfldctnt->get_proplist();
-		for (const auto tag : *pproptags)
+		for (const auto tag : pproptags)
 			pproplist->erase(tag);
 		if (!pctx->make_foldercontent(b_sub, std::move(pfldctnt)))
 			return ecError;
@@ -486,7 +479,7 @@ ec_error_t rop_fasttransfersourcecopyto(uint8_t level, uint32_t flags,
 		if (!exmdb_client->read_message_instance(plogon->get_dir(),
 		    static_cast<message_object *>(pobject)->get_instance_id(), &msgctnt))
 			return ecError;
-		for (const auto tag : *pproptags) {
+		for (const auto tag : pproptags) {
 			switch (tag) {
 			case PR_MESSAGE_RECIPIENTS:	
 				msgctnt.children.prcpts = NULL;
@@ -512,7 +505,7 @@ ec_error_t rop_fasttransfersourcecopyto(uint8_t level, uint32_t flags,
 		if (!exmdb_client->read_attachment_instance(plogon->get_dir(),
 		    static_cast<attachment_object *>(pobject)->get_instance_id(), &attctnt))
 			return ecError;
-		for (const auto tag : *pproptags) {
+		for (const auto tag : pproptags) {
 			switch (tag) {
 			case PR_ATTACH_DATA_OBJ:
 				attctnt.pembedded = NULL;
@@ -537,7 +530,7 @@ ec_error_t rop_fasttransfersourcecopyto(uint8_t level, uint32_t flags,
 }
 
 ec_error_t rop_fasttransfersourcecopyproperties(uint8_t level, uint8_t flags,
-    uint8_t send_options, const PROPTAG_ARRAY *pproptags, LOGMAP *plogmap,
+    uint8_t send_options, proptag_cspan pproptags, LOGMAP *plogmap,
     uint8_t logon_id, uint32_t hin, uint32_t *phout)
 {
 	ems_objtype object_type;
@@ -569,7 +562,7 @@ ec_error_t rop_fasttransfersourcecopyproperties(uint8_t level, uint8_t flags,
 		 * properties and subobjects to include, as opposed to
 		 * exclude""" [like rop_fasttransfersourcecopyproperties]
 		 */
-		auto bg = pproptags->begin(), end = pproptags->end();
+		auto bg = pproptags.begin(), end = pproptags.end();
 		auto b_sub    = level == 0 && std::find(bg, end, PR_CONTAINER_HIERARCHY) != end;
 		auto b_normal = level == 0 && std::find(bg, end, PR_CONTAINER_CONTENTS) != end;
 		auto b_fai    = level == 0 && std::find(bg, end, PR_FOLDER_ASSOCIATED_CONTENTS) != end;
@@ -581,7 +574,7 @@ ec_error_t rop_fasttransfersourcecopyproperties(uint8_t level, uint8_t flags,
 		auto pproplist = pfldctnt->get_proplist();
 		for (unsigned int i = 0; i < pproplist->count; ) {
 			if (pproplist->ppropval[i].proptag != MetaTagNewFXFolder) {
-				if (!pproptags->has(pproplist->ppropval[i].proptag)) {
+				if (!pproptags.has(pproplist->ppropval[i].proptag)) {
 					pproplist->erase(pproplist->ppropval[i].proptag);
 					continue;
 				}
@@ -599,16 +592,16 @@ ec_error_t rop_fasttransfersourcecopyproperties(uint8_t level, uint8_t flags,
 		    static_cast<message_object *>(pobject)->get_instance_id(), &msgctnt))
 			return ecError;
 		for (unsigned int i = 0; i < msgctnt.proplist.count; ) {
-			if (!pproptags->has(msgctnt.proplist.ppropval[i].proptag)) {
+			if (!pproptags.has(msgctnt.proplist.ppropval[i].proptag)) {
 				common_util_remove_propvals(&msgctnt.proplist,
 						msgctnt.proplist.ppropval[i].proptag);
 				continue;
 			}
 			i ++;
 		}
-		if (!pproptags->has(PR_MESSAGE_RECIPIENTS))
+		if (!pproptags.has(PR_MESSAGE_RECIPIENTS))
 			msgctnt.children.prcpts = NULL;
-		if (!pproptags->has(PR_MESSAGE_ATTACHMENTS))
+		if (!pproptags.has(PR_MESSAGE_ATTACHMENTS))
 			msgctnt.children.pattachments = NULL;
 		if (0 != level) {
 			msgctnt.children.prcpts = NULL;
@@ -624,14 +617,14 @@ ec_error_t rop_fasttransfersourcecopyproperties(uint8_t level, uint8_t flags,
 		    static_cast<attachment_object *>(pobject)->get_instance_id(), &attctnt))
 			return ecError;
 		for (unsigned int i = 0; i < attctnt.proplist.count; ) {
-			if (!pproptags->has(attctnt.proplist.ppropval[i].proptag)) {
+			if (!pproptags.has(attctnt.proplist.ppropval[i].proptag)) {
 				common_util_remove_propvals(&attctnt.proplist,
 						attctnt.proplist.ppropval[i].proptag);
 				continue;
 			}
 			i ++;
 		}
-		if (!pproptags->has(PR_ATTACH_DATA_OBJ))
+		if (!pproptags.has(PR_ATTACH_DATA_OBJ))
 			attctnt.pembedded = NULL;
 		if (!pctx->make_attachmentcontent(&attctnt))
 			return ecError;
@@ -655,8 +648,8 @@ ec_error_t rop_tellversion(const uint16_t *pversion, LOGMAP *plogmap,
 
 ec_error_t rop_syncconfigure(uint8_t sync_type, uint8_t send_options,
     uint16_t sync_flags, const RESTRICTION *pres, uint32_t extra_flags,
-    const PROPTAG_ARRAY *pproptags, LOGMAP *plogmap, uint8_t logon_id,
-    uint32_t hin, uint32_t *phout)
+    proptag_cspan pproptags, LOGMAP *plogmap, uint8_t logon_id,
+    uint32_t hin, uint32_t *phout) try
 {
 	ems_objtype object_type;
 	uint32_t permission;
@@ -686,35 +679,28 @@ ec_error_t rop_syncconfigure(uint8_t sync_type, uint8_t send_options,
 	    const_cast<RESTRICTION *>(pres)))
 			return ecError;
 
-	std::vector<proptag_t> new_tags;
-	PROPTAG_ARRAY new_pta;
-	auto bodyof = pproptags->indexof(PR_BODY);
+	proptag_vector new_tags{pproptags.begin(), pproptags.end()};
+	auto bodyit = std::find(new_tags.begin(), new_tags.end(), PR_BODY);
 	if (!(sync_flags & SYNC_ONLY_SPECIFIED_PROPS) &&
-	    bodyof != pproptags->npos && !pproptags->has(PR_HTML)) try {
+	    bodyit != new_tags.end() && !new_tags.has(PR_HTML))
 		/*
 		 * Ignore Outlook's request to exclude PR_BODY.
 		 * PR_BODY may be the only format some message has.
 		 * Send at least one body format. (Ignoring RTF presence
 		 * altogether here; that is another consideration.)
 		 */
-		auto p = pproptags->pproptag;
-		new_tags.insert(new_tags.end(), p, p + bodyof);
-		new_tags.insert(new_tags.end(), p + bodyof + 1, p + pproptags->count - 1);
-		new_pta.count = new_tags.size();
-		new_pta.pproptag = new_tags.data();
-		pproptags = &new_pta;
-	} catch (const std::bad_alloc &) {
-		mlog(LV_ERR, "%s: ENOMEM", __func__);
-		return ecServerOOM;
-	}
+		new_tags.erase(bodyit);
 	auto pctx = icsdownctx_object::create(plogon, pfolder, sync_type,
-	            send_options, sync_flags, pres, extra_flags, pproptags);
+	            send_options, sync_flags, pres, extra_flags, std::move(new_tags));
 	auto hnd = rop_processor_add_object_handle(plogmap,
 	           logon_id, hin, {ems_objtype::icsdownctx, std::move(pctx)});
 	if (hnd < 0)
 		return aoh_to_error(hnd);
 	*phout = hnd;
 	return ecSuccess;
+} catch (const std::bad_alloc &) {
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
+	return ecServerOOM;
 }
 
 /**
@@ -818,8 +804,7 @@ ec_error_t rop_syncimportmessagechange(uint8_t import_flags,
 	void *pvalue;
 	uint32_t result;
 	ems_objtype object_type;
-	uint32_t permission = rightsNone, tag_access = 0, tmp_proptag;
-	PROPTAG_ARRAY proptags;
+	uint32_t permission = rightsNone, tag_access = 0;
 	PROBLEM_ARRAY tmp_problems;
 	TPROPVAL_ARRAY tmp_propvals;
 	
@@ -908,10 +893,8 @@ ec_error_t rop_syncimportmessagechange(uint8_t import_flags,
 	if (pmessage == nullptr)
 		return ecError;
 	if (!b_new) {
-		proptags.count = 1;
-		proptags.pproptag = &tmp_proptag;
-		tmp_proptag = PR_PREDECESSOR_CHANGE_LIST;
-		if (!pmessage->get_properties(0, &proptags, &tmp_propvals))
+		static constexpr proptag_t tags[] = {PR_PREDECESSOR_CHANGE_LIST};
+		if (!pmessage->get_properties(0, tags, &tmp_propvals))
 			return ecError;
 		auto bin = tmp_propvals.get<const BINARY>(PR_PREDECESSOR_CHANGE_LIST);
 		if (bin == nullptr)
@@ -999,10 +982,8 @@ ec_error_t rop_syncimportreadstatechanges(uint16_t count,
 				continue;
 		}
 		static constexpr proptag_t proptag_buff[] = {PR_ASSOCIATED, PR_READ};
-		static constexpr PROPTAG_ARRAY tmp_proptags =
-			{std::size(proptag_buff), deconst(proptag_buff)};
 		if (!exmdb_client->get_message_properties(dir, nullptr, CP_ACP,
-		    message_id, &tmp_proptags, &tmp_propvals))
+		    message_id, proptag_cspan{proptag_buff}, &tmp_propvals))
 			return ecError;
 		auto flag = tmp_propvals.get<const uint8_t>(PR_ASSOCIATED);
 		if (flag != nullptr && *flag != 0)
@@ -1248,7 +1229,6 @@ ec_error_t rop_syncimportdeletes(uint8_t flags, const TPROPVAL_ARRAY *ppropvals,
 	XID tmp_xid;
 	void *pvalue;
 	BOOL b_exist;
-	uint64_t eid;
 	BOOL b_owner;
 	BOOL b_result;
 	BOOL b_partial;
@@ -1293,7 +1273,7 @@ ec_error_t rop_syncimportdeletes(uint8_t flags, const TPROPVAL_ARRAY *ppropvals,
 	auto pbins = static_cast<BINARY_ARRAY *>(ppropvals->ppropval[0].pvalue);
 	if (SYNC_TYPE_CONTENTS == sync_type) {
 		message_ids.count = 0;
-		message_ids.pids = cu_alloc<uint64_t>(pbins->count);
+		message_ids.pids  = cu_alloc<eid_t>(pbins->count);
 		if (message_ids.pids == nullptr)
 			return ecServerOOM;
 	}
@@ -1305,6 +1285,8 @@ ec_error_t rop_syncimportdeletes(uint8_t flags, const TPROPVAL_ARRAY *ppropvals,
 		}
 		if (!common_util_binary_to_xid(&bxid, &tmp_xid))
 			return ecError;
+
+		eid_t eid;
 		if (plogon->is_private()) {
 			auto tmp_guid = rop_util_make_user_guid(plogon->account_id);
 			if (tmp_guid != tmp_xid.guid) {
@@ -1366,7 +1348,7 @@ ec_error_t rop_syncimportdeletes(uint8_t flags, const TPROPVAL_ARRAY *ppropvals,
 			}
 		}
 		if (SYNC_TYPE_CONTENTS == sync_type) {
-			message_ids.pids[message_ids.count++] = eid;
+			message_ids.emplace_back(eid);
 		} else {
 			unsigned int empty_flags = DEL_MESSAGES | DEL_ASSOCIATED | DEL_FOLDERS;
 			empty_flags |= b_hard ? DELETE_HARD_DELETE : 0;
