@@ -126,8 +126,8 @@ static ec_error_t oxomsg_rectify_message(message_object *pmessage,
 		return ecRpcFailed;
 	return pmessage->save();
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1166: ENOMEM");
-	return ecRpcFailed;
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
+	return ecServerOOM;
 }
 
 /**
@@ -144,14 +144,12 @@ static ec_error_t oxomsg_rectify_message(message_object *pmessage,
 static bool oxomsg_extract_delegator(message_object *pmessage,
     std::string &username)
 {
-	static constexpr proptag_t proptag_buff[] =
+	static constexpr proptag_t tmp_proptags[] =
 		{PR_SENT_REPRESENTING_ADDRTYPE, PR_SENT_REPRESENTING_EMAIL_ADDRESS,
 		PR_SENT_REPRESENTING_SMTP_ADDRESS, PR_SENT_REPRESENTING_ENTRYID};
-	static constexpr PROPTAG_ARRAY tmp_proptags =
-		{std::size(proptag_buff), deconst(proptag_buff)};
 	TPROPVAL_ARRAY tmp_propvals;
 	
-	if (!pmessage->get_properties(0, &tmp_proptags, &tmp_propvals))
+	if (!pmessage->get_properties(0, tmp_proptags, &tmp_propvals))
 		return FALSE;	
 	if (0 == tmp_propvals.count) {
 		username.clear();
@@ -192,21 +190,17 @@ static bool oxomsg_extract_delegator(message_object *pmessage,
  */
 static int oxomsg_test_perm(const char *account, const char *maildir, bool send_as) try
 {
-	auto dlg_path = maildir + std::string(send_as ? "/config/sendas.txt" : "/config/delegates.txt");
 	std::vector<std::string> delegate_list;
-	auto ret = read_file_by_line(dlg_path.c_str(), delegate_list);
-	if (ret != 0 && ret != ENOENT) {
-		mlog(LV_ERR, "E-2064: %s: %s", dlg_path.c_str(), strerror(ret));
-		return ret;
-	}
+	if (!exmdb_client->read_delegates(maildir, send_as, &delegate_list))
+		return -1;
 	for (const auto &deleg : delegate_list)
 		if (strcasecmp(deleg.c_str(), account) == 0 ||
 		    mysql_adaptor_check_mlist_include(deleg.c_str(), account))
 			return 1;
 	return 0;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1500: ENOMEM");
-	return false;
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
+	return -1;
 }
 
 /**
@@ -220,6 +214,8 @@ static repr_grant oxomsg_get_perm(const char *account, const char *repr)
 	sql_meta_result mres;
 	if (mysql_adaptor_meta(repr, WANTPRIV_METAONLY, mres) != 0)
 		return repr_grant::error;
+	if (strcasecmp(account, mres.username.c_str()) == 0)
+		return repr_grant::send_as;
 	auto repdir = mres.maildir.c_str();
 	auto ret = oxomsg_test_perm(account, repdir, true);
 	if (ret < 0)
@@ -302,9 +298,8 @@ ec_error_t rop_submitmessage(uint8_t submit_flags, LOGMAP *plogmap,
 	if (rcpt_num > g_max_rcpt)
 		return ecTooManyRecips;
 
-	static constexpr proptag_t ptbuf_one[] = {PR_ASSOCIATED, PR_MESSAGE_CLASS};
-	static constexpr PROPTAG_ARRAY ptags_one = {std::size(ptbuf_one), deconst(ptbuf_one)};
-	if (!pmessage->get_properties(0, &ptags_one, &tmp_propvals))
+	static constexpr proptag_t ptags_one[] = {PR_ASSOCIATED, PR_MESSAGE_CLASS};
+	if (!pmessage->get_properties(0, ptags_one, &tmp_propvals))
 		return ecError;
 	auto flag = tmp_propvals.get<const uint8_t>(PR_ASSOCIATED);
 	/* FAI message cannot be sent */
@@ -337,11 +332,9 @@ ec_error_t rop_submitmessage(uint8_t submit_flags, LOGMAP *plogmap,
 	if (ret != ecSuccess)
 		return ret;
 	
-	static constexpr proptag_t ptbuf_two[] =
+	static constexpr proptag_t ptags_two[] =
 		{PR_MAX_SUBMIT_MESSAGE_SIZE, PR_PROHIBIT_SEND_QUOTA, PR_MESSAGE_SIZE_EXTENDED};
-	static constexpr PROPTAG_ARRAY ptags_two =
-		{std::size(ptbuf_two), deconst(ptbuf_two)};
-	if (!plogon->get_properties(&ptags_two, &tmp_propvals))
+	if (!plogon->get_properties(ptags_two, &tmp_propvals))
 		return ecError;
 
 	auto sendquota = tmp_propvals.get<uint32_t>(PR_PROHIBIT_SEND_QUOTA);
@@ -359,11 +352,8 @@ ec_error_t rop_submitmessage(uint8_t submit_flags, LOGMAP *plogmap,
 		{PR_MESSAGE_SIZE, PR_MESSAGE_FLAGS,
 		PR_DEFERRED_SEND_TIME, PR_DEFERRED_SEND_NUMBER,
 		PR_DEFERRED_SEND_UNITS, PR_DELETE_AFTER_SUBMIT};
-	PROPTAG_ARRAY tmp_proptags;
-	tmp_proptags.count = (submit_flags & ROP_SUBMIT_FLAG_NEEDS_SPOOLER) ?
-	                     2 : std::size(ptbuf_three);
-	tmp_proptags.pproptag = deconst(ptbuf_three);
-	if (!pmessage->get_properties(0, &tmp_proptags, &tmp_propvals))
+	proptag_cspan tmp_proptags = {ptbuf_three, (submit_flags & ROP_SUBMIT_FLAG_NEEDS_SPOOLER) ? 2 : std::size(ptbuf_three)};
+	if (!pmessage->get_properties(0, tmp_proptags, &tmp_propvals))
 		return ecError;
 	num = tmp_propvals.get<const uint32_t>(PR_MESSAGE_SIZE);
 	if (num == nullptr)
@@ -436,7 +426,7 @@ ec_error_t rop_submitmessage(uint8_t submit_flags, LOGMAP *plogmap,
 		pmessage->clear_unsent();
 	return ret;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2353: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ecServerOOM;
 }
 
@@ -551,10 +541,8 @@ ec_error_t rop_spoolerlockmessage(uint64_t message_id, uint8_t lock_stat,
 
 	static constexpr proptag_t proptag_buff[] =
 		{PR_DELETE_AFTER_SUBMIT, PR_TARGET_ENTRYID, PR_PARENT_ENTRYID};
-	static constexpr PROPTAG_ARRAY tmp_proptags =
-		{std::size(proptag_buff), deconst(proptag_buff)};
 	if (!exmdb_client->get_message_properties(dir, nullptr, CP_ACP,
-	    message_id, &tmp_proptags, &tmp_propvals))
+	    message_id, proptag_cspan{proptag_buff}, &tmp_propvals))
 		return ecError;
 	auto flag = tmp_propvals.get<const uint8_t>(PR_DELETE_AFTER_SUBMIT);
 	BOOL b_delete = flag != nullptr && *flag != 0 ? TRUE : false;
@@ -606,13 +594,11 @@ ec_error_t rop_transportsend(TPROPVAL_ARRAY **pppropvals, LOGMAP *plogmap,
 		return ecAccessDenied;
 	}
 
-	static constexpr uint32_t rq_tags1[] = {PR_MESSAGE_FLAGS};
-	static constexpr uint32_t cls_tags1[] = {PR_MESSAGE_CLASS};
-	static constexpr PROPTAG_ARRAY rq_tags = {1, deconst(rq_tags1)};
-	static constexpr PROPTAG_ARRAY cls_tags = {1, deconst(cls_tags1)};
+	static constexpr proptag_t rq_tags1[] = {PR_MESSAGE_FLAGS};
+	static constexpr proptag_t cls_tags[] = {PR_MESSAGE_CLASS};
 	TPROPVAL_ARRAY outvalues{};
 	if (!exmdb_client->get_message_properties(plogon->get_dir(), nullptr,
-	    CP_ACP, pmessage->get_id(), &rq_tags, &outvalues))
+	    CP_ACP, pmessage->get_id(), proptag_cspan{rq_tags1}, &outvalues))
 		return ecError;
 	auto msgflags = outvalues.get<const uint32_t>(PR_MESSAGE_FLAGS);
 	if (msgflags != nullptr && *msgflags & MSGFLAG_SUBMITTED) {
@@ -634,7 +620,7 @@ ec_error_t rop_transportsend(TPROPVAL_ARRAY **pppropvals, LOGMAP *plogmap,
 	}
 	if (repr_grant < repr_grant::send_on_behalf) {
 		TPROPVAL_ARRAY cls_vals{};
-		if (pmessage->get_properties(0, &cls_tags, &cls_vals) != 0)
+		if (!pmessage->get_properties(0, cls_tags, &cls_vals))
 			/* ignore, since we can test for cls_vals fill */;
 		auto ret = pass_scheduling("E-2080", actor, delegator.c_str(), *pmessage,
 		           cls_vals.get<const char>(PR_MESSAGE_CLASS));
@@ -647,13 +633,11 @@ ec_error_t rop_transportsend(TPROPVAL_ARRAY **pppropvals, LOGMAP *plogmap,
 		return ret;
 	*pppropvals = cu_alloc<TPROPVAL_ARRAY>();
 	if (NULL != *pppropvals) {
-		static constexpr proptag_t proptag_buff[] =
+		static constexpr proptag_t proptags[] =
 			{PR_SENDER_NAME, PR_SENDER_ENTRYID, PR_SENDER_SEARCH_KEY,
 			PR_SENT_REPRESENTING_NAME, PR_SENT_REPRESENTING_ENTRYID,
 			PR_SENT_REPRESENTING_SEARCH_KEY, PR_PROVIDER_SUBMIT_TIME};
-		static constexpr PROPTAG_ARRAY proptags =
-			{std::size(proptag_buff), deconst(proptag_buff)};
-		if (!pmessage->get_properties(0, &proptags, *pppropvals)) {
+		if (!pmessage->get_properties(0, proptags, *pppropvals)) {
 			*pppropvals = NULL;
 		} else if (!(**pppropvals).has(PR_PROVIDER_SUBMIT_TIME)) {
 			auto nt = cu_alloc<uint64_t>();
@@ -669,7 +653,7 @@ ec_error_t rop_transportsend(TPROPVAL_ARRAY **pppropvals, LOGMAP *plogmap,
 	auto ev_from = repr_grant >= repr_grant::send_as ? delegator.c_str() : actor;
 	return cu_send_message(plogon, pmessage, ev_from);
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2352: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ecServerOOM;
 }
 
