@@ -194,7 +194,7 @@ struct DELETEFOLDER_REQUEST final : public rop_request {
 
 struct SETSEARCHCRITERIA_REQUEST final : public rop_request {
 	RESTRICTION *pres;
-	LONGLONG_ARRAY folder_ids;
+	EID_ARRAY folder_ids;
 	uint32_t search_flags;
 };
 
@@ -207,13 +207,13 @@ struct GETSEARCHCRITERIA_REQUEST final : public rop_request {
 struct GETSEARCHCRITERIA_RESPONSE final : public rop_response {
 	RESTRICTION *pres;
 	uint8_t logon_id;
-	LONGLONG_ARRAY folder_ids;
+	EID_ARRAY folder_ids;
 	uint32_t search_status;
 };
 
 struct MOVECOPYMESSAGES_REQUEST final : public rop_request {
 	uint8_t dhindex;
-	LONGLONG_ARRAY message_ids;
+	EID_ARRAY message_ids;
 	uint8_t want_asynchronous;
 	uint8_t want_copy;
 };
@@ -263,13 +263,13 @@ struct HARDDELETEMESSAGESANDSUBFOLDERS_REQUEST final : public rop_request {
 struct DELETEMESSAGES_REQUEST final : public rop_request {
 	uint8_t want_asynchronous;
 	uint8_t notify_non_read;
-	LONGLONG_ARRAY message_ids;
+	EID_ARRAY message_ids;
 };
 
 struct HARDDELETEMESSAGES_REQUEST final : public rop_request {
 	uint8_t want_asynchronous;
 	uint8_t notify_non_read;
-	LONGLONG_ARRAY message_ids;
+	EID_ARRAY message_ids;
 };
 
 struct GETHIERARCHYTABLE_REQUEST final : public rop_request {
@@ -292,7 +292,7 @@ struct GETCONTENTSTABLE_RESPONSE final : public rop_response {
 
 struct SETCOLUMNS_REQUEST final : public rop_request {
 	uint8_t table_flags;
-	PROPTAG_ARRAY proptags;
+	proptag_vector proptags;
 };
 
 struct SORTTABLE_REQUEST final : public rop_request {
@@ -368,7 +368,7 @@ struct FINDROW_REQUEST final : public rop_request {
 struct FINDROW_RESPONSE final : public rop_response {
 	uint8_t bookmark_invisible;
 	PROPERTY_ROW *prow;
-	PROPTAG_ARRAY *pcolumns;
+	proptag_vector pcolumns;
 };
 
 struct FREEBOOKMARK_REQUEST final : public rop_request {
@@ -455,7 +455,7 @@ struct REMOVEALLRECIPIENTS_REQUEST final : public rop_request {
 };
 
 struct MODIFYRECIPIENTS_REQUEST final : public rop_request {
-	PROPTAG_ARRAY proptags;
+	proptag_vector proptags;
 	uint16_t count;
 	MODIFYRECIPIENT_ROW *prow;
 };
@@ -487,7 +487,7 @@ struct RELOADCACHEDINFORMATION_RESPONSE final : public rop_response {
 struct SETREADFLAGS_REQUEST final : public rop_request {
 	uint8_t want_asynchronous;
 	uint8_t read_flags;
-	LONGLONG_ARRAY message_ids;
+	EID_ARRAY message_ids;
 };
 
 struct SETMESSAGESTATUS_REQUEST final : public rop_request {
@@ -635,11 +635,11 @@ struct GETNAMESFROMPROPERTYIDS_RESPONSE final : public rop_response {
 struct GETPROPERTIESSPECIFIC_REQUEST final : public rop_request {
 	uint16_t size_limit;
 	uint16_t want_unicode;
-	PROPTAG_ARRAY proptags;
+	proptag_vector proptags;
 };
 
 struct GETPROPERTIESSPECIFIC_RESPONSE final : public rop_response {
-	PROPTAG_ARRAY *pproptags;    /* only for pushing row data into stream */
+	proptag_vector pproptags; /* only for pushing row data into stream */
 	PROPERTY_ROW row;
 };
 
@@ -665,11 +665,11 @@ struct SETPROPERTIESNOREPLICATE_REQUEST final : public rop_request {
 };
 
 struct DELETEPROPERTIES_REQUEST final : public rop_request {
-	PROPTAG_ARRAY proptags;
+	proptag_vector proptags;
 };
 
 struct DELETEPROPERTIESNOREPLICATE_REQUEST final : public rop_request {
-	PROPTAG_ARRAY proptags;
+	proptag_vector proptags;
 };
 
 struct QUERYNAMEDPROPERTIES_REQUEST final : public rop_request {
@@ -685,7 +685,7 @@ struct COPYPROPERTIES_REQUEST final : public rop_request {
 	uint8_t dhindex;
 	uint8_t want_asynchronous;
 	uint8_t copy_flags;
-	PROPTAG_ARRAY proptags;
+	proptag_vector proptags;
 };
 
 struct COPYTO_REQUEST final : public rop_request {
@@ -693,7 +693,7 @@ struct COPYTO_REQUEST final : public rop_request {
 	uint8_t want_asynchronous;
 	uint8_t want_subobjects;
 	uint8_t copy_flags;
-	PROPTAG_ARRAY excluded_proptags;
+	proptag_vector excluded_proptags;
 };
 
 struct PROGRESS_REQUEST final : public rop_request {
@@ -859,7 +859,7 @@ struct FASTTRANSFERSOURCECOPYFOLDER_REQUEST final : public rop_request {
 
 struct FASTTRANSFERSOURCECOPYMESSAGES_REQUEST final : public rop_request {
 	uint8_t ohindex;
-	LONGLONG_ARRAY message_ids;
+	EID_ARRAY message_ids;
 	uint8_t flags;
 	uint8_t send_options;
 };
@@ -869,7 +869,7 @@ struct FASTTRANSFERSOURCECOPYTO_REQUEST final : public rop_request {
 	uint8_t level;
 	uint32_t flags;
 	uint8_t send_options;
-	PROPTAG_ARRAY proptags;
+	proptag_vector proptags;
 };
 
 struct FASTTRANSFERSOURCECOPYPROPERTIES_REQUEST final : public rop_request {
@@ -877,7 +877,7 @@ struct FASTTRANSFERSOURCECOPYPROPERTIES_REQUEST final : public rop_request {
 	uint8_t level;
 	uint8_t flags;
 	uint8_t send_options;
-	PROPTAG_ARRAY proptags;
+	proptag_vector proptags;
 };
 
 struct TELLVERSION_REQUEST final : public rop_request {
@@ -891,7 +891,7 @@ struct SYNCCONFIGURE_REQUEST final : public rop_request {
 	uint16_t sync_flags;
 	RESTRICTION *pres;
 	uint32_t extra_flags;
-	PROPTAG_ARRAY proptags;
+	proptag_vector proptags;
 };
 
 struct SYNCIMPORTMESSAGECHANGE_REQUEST final : public rop_request {
