@@ -122,6 +122,8 @@ void object_node::clear() noexcept
 
 void object_node::operator=(object_node &&o) noexcept
 {
+	if (this == &o)
+		return;
 	clear();
 	type = std::move(o.type);
 	pobject = std::move(o.pobject);
@@ -314,6 +316,7 @@ logon_object *rop_processor_get_logon_object(LOGMAP *plogmap, uint8_t logon_id)
 
 static void *emsrop_scanwork(void *param)
 {
+	pthread_setname_np(pthread_self(), "rop_scan");
 	int count;
 	
 	count = 0;
@@ -355,7 +358,6 @@ int rop_processor_run()
 		       "for logon hash table: %s", strerror(ret));
 		return -5;
 	}
-	pthread_setname_np(g_scan_id, "rop_scan");
 	return 0;
 }
 
