@@ -37,6 +37,8 @@
 #include <libHX/scope.hpp>
 #include <libHX/socket.h>
 #include <libHX/string.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>
 #include <openssl/err.h>
 #if defined(OPENSSL_VERSION_NUMBER) && OPENSSL_VERSION_NUMBER >= 0x30000000L
 #	define WITH_SSLPROV 1
@@ -54,6 +56,7 @@
 #include <gromox/fileio.h>
 #include <gromox/hpm_common.h>
 #include <gromox/http.hpp>
+#include <gromox/listener_ctx.hpp>
 #include <gromox/mail_func.hpp>
 #include <gromox/mysql_adaptor.hpp>
 #include <gromox/threads_pool.hpp>
@@ -93,6 +96,10 @@ struct VIRTUAL_CONNECTION {
 };
 }
 
+enum {
+	M_UNSPECIFIED_CONN, M_UNENCRYPTED_CONN, M_TLS_CONN,
+};
+
 static constexpr time_duration OUT_CHANNEL_MAX_WAIT = std::chrono::seconds(10);
 
 namespace {
@@ -175,6 +182,7 @@ bool g_enforce_auth;
 time_duration g_http_basic_auth_validity;
 static thread_local HTTP_CONTEXT *g_context_key;
 static std::optional<http_parser> g_parser;
+static listener_ctx http_listen_ctx;
 
 static void http_parser_context_clear(HTTP_CONTEXT *pcontext);
 
@@ -2632,3 +2640,119 @@ RPC_OUT_CHANNEL::~RPC_OUT_CHANNEL()
 	}
 	double_list_free(&pdu_list);
 }
+
+int htls_thrwork(generic_connection &&conn)
+{
+	const bool use_tls = conn.mark == M_TLS_CONN;
+	char buff[1024];
+
+	if (fcntl(conn.sockd, F_SETFL, O_NONBLOCK) < 0)
+		mlog(LV_WARN, "W-1408: fcntl: %s", strerror(errno));
+	static const int flag = 1;
+	if (setsockopt(conn.sockd, IPPROTO_TCP, TCP_NODELAY, &flag, sizeof(flag)) < 0)
+		/* ignore */;
+	auto ctx = static_cast<HTTP_CONTEXT *>(contexts_pool_get_context(sctx_status::free));
+	/* there's no context available in contexts pool, close the connection*/
+	if (ctx == nullptr) {
+		mlog(LV_NOTICE, "Rejecting connection from [%s]:%hu: "
+			"reached %d connections (http.cfg:context_num)",
+			conn.client_addr, conn.client_port,
+			contexts_pool_get_param(MAX_CONTEXTS_NUM));
+		auto len = gx_snprintf(buff, std::size(buff),
+		           "HTTP/1.1 503 L-202 Service Unavailable\r\n"
+		           "Content-Length: 0\r\n"
+		           "Connection: close\r\n"
+		           "\r\n");
+		if (HXio_fullwrite(conn.sockd, buff, len) < 0)
+			mlog(LV_WARN, "W-1984: write: %s", strerror(errno));
+		return 0;
+	}
+	ctx->type = sctx_status::constructing;
+	/* pass the client ipaddr into the ipaddr filter */
+	std::string reason;
+	if (!system_services_judge_addr(conn.client_addr, reason)) {
+		auto len = gx_snprintf(buff, std::size(buff),
+		           "HTTP/1.1 503 L-216 Service Unavailable\r\n"
+		           "Content-Length: 0\r\n"
+		           "Connection: close\r\n"
+		           "\r\n");
+		if (HXio_fullwrite(conn.sockd, buff, len) < 0)
+			mlog(LV_WARN, "W-1983: write: %s", strerror(errno));
+		mlog(LV_DEBUG, "Connection %s is denied by ipaddr filter: %s",
+			conn.client_addr, reason.c_str());
+		/* release the context */
+		contexts_pool_insert(ctx, sctx_status::free);
+		return 0;
+	}
+
+	/* construct the context object */
+	ctx->connection = std::move(conn);
+	ctx->sched_stat = use_tls ? hsched_stat::initssl : hsched_stat::rdhead;
+	/*
+	 * Valid the context and wake up one thread if there are some threads
+	 * block on the condition variable
+	 */
+	ctx->polling_mask = POLLING_READ;
+	contexts_pool_insert(ctx, sctx_status::polling);
+	return 0;
+}
+
+static int http_parse_binds(listener_ctx &ctx, const config_file &gxcfg,
+    const char *gxkey, const config_file &oldcfg, const char *oldkey,
+    const char *oldportkey, unsigned int mark)
+{
+	auto line = gxcfg.get_value(gxkey);
+	if (line != nullptr)
+		return ctx.add_bunch(line, mark);
+	auto host = oldcfg.get_value(oldkey);
+	if (host != nullptr)
+		mlog(LV_NOTICE, "%s:%s is deprecated in favor of %s:%s",
+			oldcfg.m_filename.c_str(), oldkey,
+			gxcfg.m_filename.c_str(), gxkey);
+	else
+		host = "::";
+	auto ps = oldcfg.get_value(oldportkey);
+	uint16_t port = mark == M_UNENCRYPTED_CONN ? 143 : 993;
+	if (ps != nullptr) {
+		mlog(LV_NOTICE, "%s:%s is deprecated in favor of %s:%s",
+			oldcfg.m_filename.c_str(), oldportkey,
+			gxcfg.m_filename.c_str(), gxkey);
+		port = strtoul(ps, nullptr, 0);
+	}
+	if (port != 0 &&
+	    ctx.add_inet(host, port, mark) != 0)
+		return -1;
+	return 0;
+}
+
+int listener_init(const config_file &gxcfg, const config_file &oldcfg,
+    bool with_tls)
+{
+	auto &ctx = http_listen_ctx;
+	ctx.m_thread_name = "http_accept";
+	if (http_parse_binds(ctx, gxcfg, "http_listen", oldcfg,
+	    "http_listen_addr", "http_listen_port", M_UNENCRYPTED_CONN) != 0)
+		return EXIT_FAILURE;
+	if (with_tls &&
+	    http_parse_binds(ctx, gxcfg, "http_listen_tls", oldcfg,
+	    "http_listen_addr", "http_listen_tls_port", M_TLS_CONN) != 0)
+		return EXIT_FAILURE;
+	return 0;
+}
+
+int listener_trigger_accept()
+{
+	if (http_listen_ctx.empty())
+		return 0;
+	auto ret = http_listen_ctx.watch_start(g_httpmain_stop, htls_thrwork);
+	if (ret != 0) {
+		mlog(LV_ERR, "listener: failed to create listener thread: %s", strerror(ret));
+		return -1;
+	}
+	return 0;
+}
+
+void listener_stop()
+{
+	http_listen_ctx.reset();
+}
