@@ -22,6 +22,7 @@
 #include <gromox/element_data.hpp> /* MESSAGE_CONTENT alias */
 #include <gromox/fileio.h>
 #include <gromox/hpm_common.h>
+#include <gromox/idset.hpp>
 #include <gromox/mapi_types.hpp>
 #include <gromox/mysql_adaptor.hpp>
 #include <gromox/usercvt.hpp>
@@ -263,7 +264,8 @@ std::pair<unsigned int, std::string> OxdiscoPlugin::access_ok(int ctx_id,
 	    strncasecmp(target, public_folder_email, 19) == 0)
 		return {ok_code, {}};
 	unsigned int auth_user_id = 0, auth_domain_id = 0;
-	mysql_adaptor_get_user_ids(actor, &auth_user_id, &auth_domain_id, nullptr);
+	if (!mysql_adaptor_get_user_ids(actor, &auth_user_id, &auth_domain_id, nullptr))
+		return {server_error_code, server_error_msg};
 	std::vector<sql_user> hints;
 	auto err = mysql_adaptor_scndstore_hints(auth_user_id, hints);
 	if (err != 0) {
@@ -311,18 +313,18 @@ http_status OxdiscoPlugin::proc(int ctx_id, const void *content, uint64_t len) t
 	if (l == 0)
 		return http_status::none;
 	auto uri = req->f_request_uri.c_str();
-	if (auto z = umatch(uri, uri_wkmc11_xml); z != SIZE_MAX) {
-		if (uri[z] != '?')
+	if (auto z0 = umatch(uri, uri_wkmc11_xml); z0 != SIZE_MAX) {
+		if (uri[z0] != '?')
 			return resp_autocfg(ctx_id, auth_info.username);
 		auto username = extract_qparam(&uri[45], "emailaddress");
 		return resp_autocfg(ctx_id, username.c_str());
-	} else if (auto z = umatch(uri, uri_wkcaldav); z != SIZE_MAX && uri[z] != '/') {
+	} else if (auto z1 = umatch(uri, uri_wkcaldav); z1 != SIZE_MAX && uri[z1] != '/') {
 		/* Not using mod_rewrite: Silent rewrite disallowed by RFC */
 		return resp_dav(ctx_id);
-	} else if (auto z = umatch(uri, uri_wkcarddav); z != SIZE_MAX && uri[z] != '/') {
+	} else if (auto z2 = umatch(uri, uri_wkcarddav); z2 != SIZE_MAX && uri[z2] != '/') {
 		return resp_dav(ctx_id);
-	} else if (auto z = umatch(uri, uri_mc11_xml); z != SIZE_MAX) {
-		if (uri[z] != '?')
+	} else if (auto z3 = umatch(uri, uri_mc11_xml); z3 != SIZE_MAX) {
+		if (uri[z3] != '?')
 			return resp_autocfg(ctx_id, auth_info.username);
 		auto username = extract_qparam(&uri[45], "emailaddress");
 		return resp_autocfg(ctx_id, username.c_str());
@@ -672,7 +674,8 @@ int OxdiscoPlugin::resp_web(XMLElement *el, const char *authuser,
 			mlog(LV_ERR, "oxdisco: could not obtain PR_DISPLAY_NAME for \"%s\"", email);
 			return -1;
 		}
-		mysql_adaptor_get_user_ids(email, &user_id, &domain_id, nullptr);
+		if (!mysql_adaptor_get_user_ids(email, &user_id, &domain_id, nullptr))
+			return -1;
 		if (cvt_username_to_essdn(email, x500_org_name.c_str(),
 		    mysql_adaptor_get_user_ids, mysql_adaptor_get_domain_ids,
 		    essdn) != ecSuccess)
