@@ -37,12 +37,12 @@ struct container_object {
 	~container_object() { clear(); }
 	static std::unique_ptr<container_object> create(uint8_t type, CONTAINER_ID);
 	void clear();
-	BOOL get_properties(const PROPTAG_ARRAY *, TPROPVAL_ARRAY *);
+	bool get_properties(proptag_cspan, TPROPVAL_ARRAY *);
 	BOOL load_user_table(const RESTRICTION *);
 	BOOL get_container_table_num(BOOL depth, uint32_t *num);
-	BOOL query_container_table(const PROPTAG_ARRAY *, BOOL depth, uint32_t start_pos, int32_t row_needed, TARRAY_SET *);
+	bool query_container_table(proptag_cspan, BOOL depth, uint32_t start_pos, int32_t row_needed, TARRAY_SET *);
 	BOOL get_user_table_num(uint32_t *);
-	BOOL query_user_table(const PROPTAG_ARRAY *, uint32_t start_pos, int32_t row_needed, TARRAY_SET *);
+	bool query_user_table(proptag_cspan, uint32_t start_pos, int32_t row_needed, TARRAY_SET *);
 
 	uint8_t type = 0;
 	CONTAINER_ID id{};
@@ -60,9 +60,9 @@ struct folder_object {
 	static std::unique_ptr<folder_object> create(store_object *, uint64_t folder_id, uint8_t type, uint32_t tag_access);
 	BOOL get_all_proptags(PROPTAG_ARRAY *);
 	bool is_readonly_prop(gromox::proptag_t) const;
-	BOOL get_properties(const PROPTAG_ARRAY *, TPROPVAL_ARRAY *);
+	bool get_properties(proptag_cspan, TPROPVAL_ARRAY *);
 	BOOL set_properties(const TPROPVAL_ARRAY *);
-	BOOL remove_properties(const PROPTAG_ARRAY *);
+	bool remove_properties(proptag_cspan);
 	BOOL get_permissions(PERMISSION_SET *);
 	BOOL set_permissions(const PERMISSION_SET *);
 	BOOL updaterules(uint32_t flags, RULE_LIST *);
@@ -136,11 +136,11 @@ struct message_object {
 	bool importing() const { return message_id != 0 && pstate != nullptr; }
 	bool writable() const { return b_writable; }
 	gromox::errno_t init_message(bool fai, cpid_t);
-	uint64_t get_id() const { return message_id; }
+	eid_t get_id() const { return message_id; }
 	store_object *get_store() const { return pstore; }
 	ec_error_t save();
 	BOOL reload();
-	BOOL write_message(const message_content *);
+	bool write_message(const message_content &);
 	BOOL get_recipient_all_proptags(PROPTAG_ARRAY *);
 	BOOL read_recipients(uint32_t row_id, uint16_t need_count, TARRAY_SET *);
 	BOOL get_rowid_begin(uint32_t *begin_id);
@@ -150,18 +150,19 @@ struct message_object {
 	BOOL get_attachments_num(uint16_t *);
 	BOOL delete_attachment(uint32_t attachment_num);
 	BOOL get_attachment_table_all_proptags(PROPTAG_ARRAY *);
-	BOOL query_attachment_table(const PROPTAG_ARRAY *, uint32_t start_pos, int32_t row_needed, TARRAY_SET *);
+	bool query_attachment_table(proptag_cspan, uint32_t start_pos, int32_t row_needed, TARRAY_SET *);
 	BOOL clear_unsent();
 	BOOL get_all_proptags(PROPTAG_ARRAY *);
-	BOOL get_properties(const PROPTAG_ARRAY *, TPROPVAL_ARRAY *);
+	bool get_properties(proptag_cspan, TPROPVAL_ARRAY *);
 	BOOL set_properties(TPROPVAL_ARRAY *);
-	BOOL remove_properties(const PROPTAG_ARRAY *);
-	BOOL copy_to(message_object *src, const PROPTAG_ARRAY *exclprop, BOOL force, BOOL *cycle);
+	bool remove_properties(proptag_cspan);
+	bool copy_to(message_object *src, proptag_cspan exclprop, BOOL force, BOOL *cycle);
 	BOOL set_readflag(uint8_t read_flag, BOOL *changed);
 
 	store_object *pstore = nullptr;
 	BOOL b_new = false, b_writable = false, b_touched = false;
-	uint64_t change_num = 0, message_id = 0, folder_id = 0;
+	uint64_t change_num = 0;
+	eid_t message_id{}, folder_id{};
 	cpid_t cpid = CP_ACP;
 	uint32_t instance_id = 0, tag_access = 0;
 	attachment_object *pembedding = nullptr;
@@ -185,10 +186,10 @@ struct attachment_object {
 	uint32_t get_tag_access() const { return pparent->tag_access; }
 	ec_error_t save();
 	BOOL get_all_proptags(PROPTAG_ARRAY *);
-	BOOL get_properties(const PROPTAG_ARRAY *, TPROPVAL_ARRAY *);
+	bool get_properties(proptag_cspan, TPROPVAL_ARRAY *);
 	BOOL set_properties(const TPROPVAL_ARRAY *);
-	BOOL remove_properties(const PROPTAG_ARRAY *);
-	BOOL copy_properties(attachment_object *src, const PROPTAG_ARRAY *exclprop, BOOL force, BOOL *cycle);
+	bool remove_properties(proptag_cspan);
+	bool copy_properties(attachment_object *src, proptag_cspan exclprop, BOOL force, BOOL *cycle);
 	store_object *get_store() const { return pparent->pstore; }
 	bool writable() const { return b_writable; }
 
@@ -204,9 +205,9 @@ struct user_object {
 	public:
 	static std::unique_ptr<user_object> create(int base_id, uint32_t minid);
 	bool valid();
-	BOOL get_properties(const PROPTAG_ARRAY *, TPROPVAL_ARRAY *);
+	bool get_properties(proptag_cspan, TPROPVAL_ARRAY *);
 	ec_error_t load_list_members(const RESTRICTION *);
-	ec_error_t query_member_table(const PROPTAG_ARRAY *, uint32_t start_pos, int32_t row_needed, TARRAY_SET *);
+	ec_error_t query_member_table(proptag_cspan, uint32_t start_pos, int32_t row_needed, TARRAY_SET *);
 
 	int base_id = 0;
 	uint32_t minid = 0;
@@ -219,7 +220,7 @@ struct oneoff_object {
 
 	public:
 	static std::unique_ptr<oneoff_object> create(ONEOFF_ENTRYID &&);
-	ec_error_t get_props(const PROPTAG_ARRAY *, TPROPVAL_ARRAY *);
+	ec_error_t get_props(proptag_cspan, TPROPVAL_ARRAY *);
 
 	static const gromox::proptag_t all_tags_raw[];
 	static const PROPTAG_ARRAY all_tags;
