@@ -10,8 +10,9 @@
 #include <type_traits>
 #include <unordered_map>
 #include <variant>
-#include <gromox/mysql_adaptor.hpp>
 #include <gromox/clock.hpp>
+#include <gromox/mapidefs.h>
+#include <gromox/mysql_adaptor.hpp>
 
 namespace gromox::ab_tree {
 
@@ -160,7 +161,7 @@ class GX_EXPORT ab_base {
 		iterator(const ab_base *b, const std::vector<ab_domain>::const_iterator &i) :
 			m_base(b), it(i)
 		{
-			if (i != m_base->domains.cend())
+			if (i != m_base->m_domains.cend())
 				mid = minid(minid::domain, i->id);
 			else
 				mid = 0;
@@ -228,7 +229,7 @@ class GX_EXPORT ab_base {
 	minid at(uint32_t) const;
 	const std::vector<std::string> &aliases(minid) const;
 	size_t children_count(minid) const;
-	bool company_info(minid, std::string *, std::string *) const;
+	bool company_name(minid, std::string &) const;
 	std::string displayname(minid) const;
 	bool dn(minid, std::string &) const;
 	uint32_t dtyp(minid) const;
@@ -238,7 +239,7 @@ class GX_EXPORT ab_base {
 	bool exists(minid) const;
 	const ab_domain *fetch_domain(minid) const;
 	ec_error_t fetch_prop(minid, gromox::proptag_t, std::string &) const;
-	bool fetch_props(minid, const PROPTAG_ARRAY &, std::unordered_map<gromox::proptag_t, std::string> &) const;
+	bool fetch_props(minid, proptag_cspan, std::unordered_map<gromox::proptag_t, std::string> &) const;
 	const sql_user *fetch_user(minid) const;
 	uint32_t get_leaves_num(minid) const;
 	inline const GUID &guid() const { return m_guid; }
@@ -246,16 +247,17 @@ class GX_EXPORT ab_base {
 	uint32_t hidden(minid) const;
 	ec_error_t mdbdn(minid, std::string &) const;
 	bool mlist_info(minid, std::string *, std::string *, int *) const;
+	bool office_location(minid, std::string &) const;
 	ec_error_t proplist(minid, std::vector<gromox::proptag_t> &) const;
 	minid resolve(const char *) const;
-	inline size_t size() const { return m_users.size() + domains.size(); }
+	inline size_t size() const { return m_users.size() + m_domains.size(); }
 	abnode_type type(minid) const;
 	inline size_t user_count() const { return m_users.size(); }
 	const char *user_info(minid, userinfo) const;
 
-	inline iterator begin() const { return iterator(this, domains.cbegin()); } ///< Iterator to beginning of address book
+	inline iterator begin() const { return iterator(this, m_domains.cbegin()); } ///< Iterator to beginning of address book
 	inline iterator end() const { return iterator(this, m_users.cend()); } ///< Iterator to end of address book
-	inline iterator dbegin() const { return iterator(this, domains.cbegin()); } ///< Iterator to beginning of domain list
+	inline iterator dbegin() const { return iterator(this, m_domains.cbegin()); } ///< Iterator to beginning of domain list
 	inline iterator dend() const { return iterator(this, m_users.cbegin()); } ///< Iterator to end of domain list
 	inline iterator ubegin() const { return iterator(this, m_users.cbegin()); } ///< Iterator to beginning of user list
 	inline iterator uend() const { return iterator(this, m_users.cend()); } ///< Iterator to end of user list
@@ -281,7 +283,7 @@ class GX_EXPORT ab_base {
 	 *             @domains will have exactly one entry.
 	 */
 	int m_base_id = 0;
-	std::vector<ab_domain> domains; ///< list of domains belonging to the base
+	std::vector<ab_domain> m_domains; ///< list of domains belonging to the base
 	std::vector<sql_user> m_users; ///< list of users from all those domains, sorted by displayname
 	std::vector<minid> filtered_gal;
 	std::unordered_map<minid, uint32_t> minid_idx_map; ///< map from minid to index in domain/user list
@@ -353,7 +355,7 @@ struct GX_EXPORT ab_node {
 
 	WRAP(aliases)
 	WRAP(children_count)
-	WRAP(company_info)
+	WRAP(company_name)
 	WRAP(displayname)
 	WRAP(dn)
 	WRAP(dtyp)
@@ -365,6 +367,7 @@ struct GX_EXPORT ab_node {
 	WRAP(get_leaves_num)
 	WRAP(mdbdn)
 	WRAP(mlist_info)
+	WRAP(office_location)
 	WRAP(proplist)
 	WRAP(type)
 	WRAP(user_info)
