@@ -9,4 +9,4 @@
 using namespace gromox;
 
 extern GX_EXPORT unsigned int freebusy_perms(const char *actor, const char *target);
-extern GX_EXPORT bool get_freebusy(const char *, const char *, time_t, time_t, std::vector<freebusy_event> &);
+extern GX_EXPORT ec_error_t get_freebusy(const char *, const char *, time_t, time_t, std::vector<freebusy_event> &);
