@@ -313,25 +313,6 @@ static BOOL list_file_parse_line(LIST_FILE* list_file, char* pfile, char* line)
 	return b_terminate;
 }
 
-errno_t list_file_read_fixedstrings(const char *filename, const char *sdlist,
-    std::vector<std::string> &out)
-{
-	struct item { char data[256]; };
-	auto plist = list_file_initd(filename, sdlist, "%s:256", ERROR_ON_ABSENCE);
-	if (plist == nullptr)
-		return errno;
-	auto num = plist->get_size();
-	auto pitem = static_cast<const item *>(plist->get_list());
-	for (decltype(num) i = 0; i < num; ++i) {
-		try {
-			out.emplace_back(pitem[i].data);
-		} catch (const std::bad_alloc &) {
-			return ENOMEM;
-		}
-	}
-	return 0;
-}
-
 errno_t list_file_read_exmdb(const char *filename, const char *sdlist,
     std::vector<EXMDB_ITEM> &out)
 {
