@@ -10,6 +10,10 @@ enum {
     FLUSHER_MODE_GATEWAY    
 };
 
+enum {
+	M_UNSPECIFIED_CONN, M_UNENCRYPTED_CONN, M_TLS_CONN,
+};
+
 class config_file;
 struct FLUSH_ENTITY;
 struct smtp_context;
@@ -26,8 +30,6 @@ extern BOOL flusher_register_cancel(CANCEL_FUNCTION);
 extern void flusher_set_flush_ID(int);
 extern BOOL FLH_LibMain(enum plugin_op);
 
-extern uint16_t g_listener_ssl_port;
-
 extern int resource_run();
 extern void resource_stop();
 extern const char *resource_get_smtp_code(unsigned int code_type, unsigned int n, size_t *len);
