@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: AGPL-3.0-or-later
-// SPDX-FileCopyrightText: 2023–2025 grommunio GmbH
+// SPDX-FileCopyrightText: 2023–2026 grommunio GmbH
 // This file is part of Gromox.
 #include <algorithm>
 #include <cerrno>
@@ -17,7 +17,6 @@
 #include <libHX/scope.hpp>
 #include <gromox/ext_buffer.hpp>
 #include <gromox/mapitags.hpp>
-#include <gromox/paths.h>
 #include <gromox/propval.hpp>
 #include <gromox/textmaps.hpp>
 #include <gromox/tie.hpp>
@@ -216,7 +215,7 @@ static void read_col(libesedb_record_t *row, unsigned int x,
 			throw az_error("EE-1019", err);
 		if (dsize64 == 0)
 			return;
-		dsize = std::max(dsize64, static_cast<uint64_t>(UINT32_MAX));
+		dsize = std::min(dsize64, static_cast<uint64_t>(UINT32_MAX));
 		udata.resize(dsize);
 		if (libesedb_long_value_get_data(lv.get(),
 		    TOU8(udata.data()), dsize, &~unique_tie(err)) < 1)
@@ -538,7 +537,7 @@ static void do_folder(mbox_state &mbs, const edb_folder &folder)
  */
 static errno_t do_mbox(mbox_state &mbs)
 {
-	if (HXio_fullwrite(STDOUT_FILENO, "GXMT0004", 8) < 0)
+	if (HXio_fullwrite(STDOUT_FILENO, "GXMT0005", 8) < 0)
 		throw YError("PG-1014: %s", strerror(errno));
 	uint8_t flag = false;
 	if (HXio_fullwrite(STDOUT_FILENO, &flag, sizeof(flag)) < 0) /* splice flag */
@@ -626,7 +625,7 @@ static errno_t do_file(const char *filename) try
 		return EXIT_FAILURE;
 	}
 	if (ep.p_uint32(static_cast<uint32_t>(MAPI_MESSAGE)) != pack_result::ok ||
-	    ep.p_uint32(1) != pack_result::ok ||
+	    ep.p_uint64(1) != pack_result::ok ||
 	    ep.p_uint32(static_cast<uint32_t>(parent.type)) != pack_result::ok ||
 	    ep.p_uint64(parent.folder_id) != pack_result::ok ||
 	    ep.p_msgctnt(*ctnt) != pack_result::ok ||
@@ -685,9 +684,10 @@ int main(int argc, char **argv)
 	}
 #endif
 	mlog_init(nullptr, nullptr, g_mlog_level, nullptr);
+	setup_utf8_locale();
 	if (iconv_validate() != 0)
 		return EXIT_FAILURE;
-	textmaps_init(PKGDATADIR);
+	textmaps_init();
 
 	auto ret = do_file(argp.uarg[0]);
 	if (ret != 0) {
