@@ -1,3 +1,4 @@
+#include "mbop_cvt.cpp"
 #include "mbop_delmsg.cpp"
 #include "mbop_emptyfld.cpp"
 #include "mbop_foreach.cpp"
@@ -5,3 +6,4 @@
 #include "mbop_locale.cpp"
 #include "mbop_main.cpp"
 #include "mbop_purge.cpp"
+#include "mbop_walk.cpp"
